@@ -1,8 +1,4 @@
-use std::{
-    collections::{hash_map, HashMap},
-    iter,
-    net::Ipv4Addr,
-};
+use std::{collections::HashMap, iter, net::Ipv4Addr};
 
 use crate::{
     decoder::{Decodable, Decoder},
@@ -34,11 +30,11 @@ impl DhcpOptions {
         self.0.insert((&opt).into(), opt)
     }
     /// iterate over entries
-    pub fn iter(&self) -> hash_map::Iter<'_, OptionCode, DhcpOption> {
+    pub fn iter(&self) -> impl Iterator<Item = (&OptionCode, &DhcpOption)> {
         self.0.iter()
     }
     /// iterate mutably over entries
-    pub fn iter_mut(&mut self) -> hash_map::IterMut<'_, OptionCode, DhcpOption> {
+    pub fn iter_mut(&mut self) -> impl Iterator<Item = (&OptionCode, &mut DhcpOption)> {
         self.0.iter_mut()
     }
     /// return message type
@@ -70,6 +66,7 @@ impl Decodable for DhcpOptions {
                 DhcpOption::End => {
                     break;
                 }
+                DhcpOption::Pad => {}
                 _ => {
                     opts.insert(OptionCode::from(&opt), opt);
                 }
@@ -219,6 +216,8 @@ pub enum OptionCode {
     ClientIdentifier,
     /// 82 Relay Agent Information
     RelayAgentInformation,
+    /// 118 Subnet option - <https://datatracker.ietf.org/doc/html/rfc3011>
+    SubnetSelection,
     /// Unknown option
     Unknown(u8),
     /// 255 End
@@ -288,6 +287,7 @@ impl From<u8> for OptionCode {
             60 => ClassIdentifier,
             61 => ClientIdentifier,
             82 => RelayAgentInformation,
+            118 => SubnetSelection,
             255 => End,
             // TODO: implement more
             n => Unknown(n),
@@ -358,6 +358,7 @@ impl From<OptionCode> for u8 {
             ClassIdentifier => 60,
             ClientIdentifier => 61,
             RelayAgentInformation => 82,
+            SubnetSelection => 118,
             End => 255,
             // TODO: implement more
             Unknown(n) => n,
@@ -491,6 +492,8 @@ pub enum DhcpOption {
     ClientIdentifier(Vec<u8>),
     /// 82 Relay Agent Information - <https://datatracker.ietf.org/doc/html/rfc3046>
     RelayAgentInformation(relay::RelayAgentInformation),
+    /// 118 Subnet selection - <https://datatracker.ietf.org/doc/html/rfc3011>
+    SubnetSelection(Ipv4Addr),
     /// Unknown option
     Unknown(UnknownOption),
     /// 255 End
@@ -785,12 +788,16 @@ impl Decodable for DhcpOption {
                 let mut dec = Decoder::new(decoder.read_slice(length as usize)?);
                 RelayAgentInformation(relay::RelayAgentInformation::decode(&mut dec)?)
             }
+            OptionCode::SubnetSelection => {
+                let len = decoder.read_u8()?; // always 4
+                SubnetSelection(decoder.read_ipv4(len as usize)?)
+            }
             OptionCode::End => End,
             // not yet implemented
             OptionCode::Unknown(code) => {
                 let length = decoder.read_u8()?;
                 let bytes = decoder.read_slice(length as usize)?.to_vec();
-                Unknown(UnknownOption { code, bytes })
+                Unknown(UnknownOption { code, data: bytes })
             }
         })
     }
@@ -814,7 +821,8 @@ impl Encodable for DhcpOption {
             | BroadcastAddr(addr)
             | RouterSolicitationAddr(addr)
             | RequestedIpAddress(addr)
-            | ServerIdentifier(addr) => {
+            | ServerIdentifier(addr)
+            | SubnetSelection(addr) => {
                 e.write_u8(code.into())?;
                 e.write_u8(4)?;
                 e.write_u32((*addr).into())?
@@ -925,8 +933,8 @@ impl Encodable for DhcpOption {
             Unknown(opt) => {
                 e.write_u8(code.into())?;
                 // length of bytes stored in Vec
-                e.write_u8(opt.bytes.len() as u8)?;
-                e.write_slice(&opt.bytes)?
+                e.write_u8(opt.data.len() as u8)?;
+                e.write_slice(&opt.data)?
             }
         };
         Ok(())
@@ -995,6 +1003,7 @@ impl From<&DhcpOption> for OptionCode {
             ClassIdentifier(_) => OptionCode::ClassIdentifier,
             ClientIdentifier(_) => OptionCode::ClientIdentifier,
             RelayAgentInformation(_) => OptionCode::RelayAgentInformation,
+            SubnetSelection(_) => OptionCode::SubnetSelection,
             End => OptionCode::End,
             // TODO: implement more
             Unknown(n) => OptionCode::Unknown(n.code),
@@ -1006,7 +1015,28 @@ impl From<&DhcpOption> for OptionCode {
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct UnknownOption {
     code: u8,
-    bytes: Vec<u8>,
+    data: Vec<u8>,
+}
+
+impl UnknownOption {
+    pub fn new(code: OptionCode, data: Vec<u8>) -> Self {
+        Self {
+            code: code.into(),
+            data,
+        }
+    }
+    /// return the relay code
+    pub fn code(&self) -> OptionCode {
+        self.code.into()
+    }
+    /// return the data for this code
+    pub fn data(&self) -> &[u8] {
+        &self.data
+    }
+    /// take ownership and return the parts of this
+    pub fn into_parts(self) -> (OptionCode, Vec<u8>) {
+        (self.code.into(), self.data)
+    }
 }
 
 /// The DHCP message type
@@ -1158,7 +1188,7 @@ mod tests {
         test_opt(
             DhcpOption::Unknown(UnknownOption {
                 code: 91,
-                bytes: vec![1, 2, 3, 4],
+                data: vec![1, 2, 3, 4],
             }),
             vec![91, 4, 1, 2, 3, 4],
         )?;
